@@ -16,25 +16,36 @@ use crate::{ffi, AsColumnFamilyRef};
 use libc::{c_char, c_void, size_t};
 use std::slice;
 
+/// A type alias to keep compatibility. See [`WriteBatchWithTransaction`] for details
+pub type WriteBatch = WriteBatchWithTransaction<false>;
+
 /// An atomic batch of write operations.
 ///
+/// [`delete_range`](#method.delete_range) is not supported in [`Transaction`].
+///
 /// Making an atomic commit of several writes:
 ///
 /// ```
-/// use rocksdb::{DB, Options, WriteBatch};
+/// use rocksdb::{DB, Options, WriteBatchWithTransaction};
 ///
 /// let path = "_path_for_rocksdb_storage1";
 /// {
 ///     let db = DB::open_default(path).unwrap();
-///     let mut batch = WriteBatch::default();
+///     let mut batch = WriteBatchWithTransaction::<false>::default();
 ///     batch.put(b"my key", b"my value");
 ///     batch.put(b"key2", b"value2");
 ///     batch.put(b"key3", b"value3");
+///
+///     // delete_range is supported when use without transaction
+///     batch.delete_range(b"key2", b"key3");
+///
 ///     db.write(batch); // Atomically commits the batch
 /// }
 /// let _ = DB::destroy(&Options::default(), path);
 /// ```
-pub struct WriteBatch {
+///
+/// [`Transaction`]: crate::Transaction
+pub struct WriteBatchWithTransaction<const TRANSACTION: bool> {
     pub(crate) inner: *mut ffi::rocksdb_writebatch_t,
 }
 
@@ -60,8 +71,8 @@ unsafe extern "C" fn writebatch_put_callback(
     // freeing the resource before we are done with it
     let boxed_cb = Box::from_raw(state as *mut &mut dyn WriteBatchIterator);
     let leaked_cb = Box::leak(boxed_cb);
-    let key = slice::from_raw_parts(k as *const u8, klen as usize);
-    let value = slice::from_raw_parts(v as *const u8, vlen as usize);
+    let key = slice::from_raw_parts(k as *const u8, klen);
+    let value = slice::from_raw_parts(v as *const u8, vlen);
     leaked_cb.put(
         key.to_vec().into_boxed_slice(),
         value.to_vec().into_boxed_slice(),
@@ -73,11 +84,25 @@ unsafe extern "C" fn writebatch_delete_callback(state: *mut c_void, k: *const c_
     // freeing the resource before we are done with it
     let boxed_cb = Box::from_raw(state as *mut &mut dyn WriteBatchIterator);
     let leaked_cb = Box::leak(boxed_cb);
-    let key = slice::from_raw_parts(k as *const u8, klen as usize);
+    let key = slice::from_raw_parts(k as *const u8, klen);
     leaked_cb.delete(key.to_vec().into_boxed_slice());
 }
 
-impl WriteBatch {
+impl<const TRANSACTION: bool> WriteBatchWithTransaction<TRANSACTION> {
+    /// Construct with a reference to a byte array serialized by [`WriteBatch`].
+    pub fn from_data(data: &[u8]) -> Self {
+        unsafe {
+            let ptr = data.as_ptr();
+            let len = data.len();
+            Self {
+                inner: ffi::rocksdb_writebatch_create_from(
+                    ptr as *const libc::c_char,
+                    len as size_t,
+                ),
+            }
+        }
+    }
+
     pub fn len(&self) -> usize {
         unsafe { ffi::rocksdb_writebatch_count(self.inner) as usize }
     }
@@ -87,7 +112,16 @@ impl WriteBatch {
         unsafe {
             let mut batch_size: size_t = 0;
             ffi::rocksdb_writebatch_data(self.inner, &mut batch_size);
-            batch_size as usize
+            batch_size
+        }
+    }
+
+    /// Return a reference to a byte array which represents a serialized version of the batch.
+    pub fn data(&self) -> &[u8] {
+        unsafe {
+            let mut batch_size: size_t = 0;
+            let batch_data = ffi::rocksdb_writebatch_data(self.inner, &mut batch_size);
+            std::slice::from_raw_parts(batch_data as _, batch_size)
         }
     }
 
@@ -110,7 +144,7 @@ impl WriteBatch {
             );
             // we must manually set the raw box free since there is no
             // associated "destroy" callback for this object
-            Box::from_raw(state);
+            drop(Box::from_raw(state));
         }
     }
 
@@ -219,6 +253,15 @@ impl WriteBatch {
         }
     }
 
+    /// Clear all updates buffered in this batch.
+    pub fn clear(&mut self) {
+        unsafe {
+            ffi::rocksdb_writebatch_clear(self.inner);
+        }
+    }
+}
+
+impl WriteBatchWithTransaction<false> {
     /// Remove database entries from start key to end key.
     ///
     /// Removes the database entries in the range ["begin_key", "end_key"), i.e.,
@@ -257,16 +300,9 @@ impl WriteBatch {
             );
         }
     }
-
-    /// Clear all updates buffered in this batch.
-    pub fn clear(&mut self) {
-        unsafe {
-            ffi::rocksdb_writebatch_clear(self.inner);
-        }
-    }
 }
 
-impl Default for WriteBatch {
+impl<const TRANSACTION: bool> Default for WriteBatchWithTransaction<TRANSACTION> {
     fn default() -> Self {
         Self {
             inner: unsafe { ffi::rocksdb_writebatch_create() },
@@ -274,7 +310,7 @@ impl Default for WriteBatch {
     }
 }
 
-impl Drop for WriteBatch {
+impl<const TRANSACTION: bool> Drop for WriteBatchWithTransaction<TRANSACTION> {
     fn drop(&mut self) {
         unsafe {
             ffi::rocksdb_writebatch_destroy(self.inner);
@@ -282,4 +318,4 @@ impl Drop for WriteBatch {
     }
 }
 
-unsafe impl Send for WriteBatch {}
+unsafe impl<const TRANSACTION: bool> Send for WriteBatchWithTransaction<TRANSACTION> {}
